function residue = assign_resSeq2refseq(pdbStr, refSeq, domain, location, manualalignment, jobname)
% input: two pdb structure (matlab) and the reference sequence from MSA
%        domain boundaries of interest and their gap free locations on the MSA
% output: the residue numbers for each residue on the refSeq

% the sequence will be extracted from the pdb file only for the chains that
% specified in the domain, and then do the alignment with the corresponding
% sequence in MSA
clear sequence;
 
for dk = 1 : 2
    sequence{dk}  = [];
    if size(location{dk},1) > 1
        % for non-continuous domains
        % our strategy is use the continuous sequence for the residue
        % number assignment firstly
         sequence{dk}  = [sequence{dk}, refSeq( [location{dk}(1,1) : location{dk}(size(location{dk},1),2)] )];        
    else
        % for continuous domains
        sequence{dk} =  refSeq( [location{dk}(1,1) : location{dk}(1,2)] );
    end
end
clear pdbSeq;
clear pdbRes;
for i = 1 : 2
    % go through the pdb structure to find the residues specified by domain
    clear Atom;
    Atom = pdbStr{i}.Model.Atom;
    % find the first eligible residue
    for j = 1 : size(Atom,2)
        if strcmpi(Atom(j).chainID, domain.chain{i}) ... % in the chain specified
                && myaminolookup(Atom(j).resName) ~= 0 ... % stardard amino acid
                && size(Atom(j).iCode,2) == 0 % not insertion of residue
            pdbRes{i}.number = Atom(j).resSeq;
            pdbRes{i}.chain = Atom(j).chainID;
            pdbSeq{i} = myaminolookup(Atom(j).resName);
            break;
        end
    end
    % extract all sequence on the chain
    for j = 1 : size(Atom,2)
        if strcmpi(Atom(j).chainID, domain.chain{i}) ... % in the chain specified
                && myaminolookup(Atom(j).resName) ~= 0 ... % stardard amino acid
                && size(Atom(j).iCode,2) == 0 % not insertion of residue
            if ~isin(Atom(j).resSeq, pdbRes{i}.number)
                pdbRes{i}.number = [pdbRes{i}.number, Atom(j).resSeq];
                pdbRes{i}.chain = [pdbRes{i}.chain, Atom(j).chainID];
                pdbSeq{i} = [pdbSeq{i}, myaminolookup(Atom(j).resName)];
            end
        end
    end 
    %% local alignment
    clear Alignment;
    [Score, Alignment] = swalign(pdbSeq{i},sequence{i});
    %% put back the deleted residues
    if manualalignment == 0
        Alignment = expandlocalAln(pdbSeq{i}, sequence{i}, Alignment);
    else
        fid = fopen([jobname, 'myAlignment.mat']);
        if fid > 0 
            clear temp;
            temp = load([jobname, 'myAlignment.mat']);
            Alignment = temp.allAln{i};
            clear temp;
            fclose(fid);
        else
            'please make the alginment between the reference sequence and the pdb sequence. stop the algorithm by ctrl+c'
            pause();
        end
    end
    %% resNum Check 2: check the residues in the pdbSeq if any two residues
    %% with adjacent resNum's in pdb file but separated by space in the
    %% alignment. If so, we need to modify the alginment to make those
    %% adjacent residues next to each other
    clear residueposition;
    residueposition = find(Alignment(1,:) ~= '-');
    key = 0;
    for alk = 1 : size(residueposition,2)-1
        if Alignment(2,residueposition(alk)) == '|'
            key = 1;
        end
        if key == 1
            if (residueposition(alk+1) - residueposition(alk)) ...
                    > (pdbRes{i}.number(alk+1) - pdbRes{i}.number(alk))
                % something wrong in the alignment, need to move the position
                % of the alk+1 residue to the right position which apart
                % from the alk-th residue for  (residueposition(alk+1) -
                % residueposition(alk)) gaps
                mgaps =  (pdbRes{i}.number(alk+1) - pdbRes{i}.number(alk));
                if Alignment(1,residueposition(alk)+mgaps) == '-' 
                    Alignment(1, residueposition(alk)+mgaps) = Alignment(1, residueposition(alk+1));
                    if Alignment(3,residueposition(alk)+mgaps) == Alignment(1, residueposition(alk)+mgaps)
                        Alignment(2, residueposition(alk)+mgaps) = '|';
                    else
                        Alignment(2, residueposition(alk)+mgaps) = ':';
                    end
                    Alignment(1, residueposition(alk+1)) = '-';
                    if Alignment(3, residueposition(alk+1)) ~= '-'
                        Alignment(2, residueposition(alk+1)) = ' ';
                    else
                        clear temp;
                        temp = Alignment(:,[1:residueposition(alk+1)-1,residueposition(alk+1)+1:end]);
                        clear Alignment;
                        Alignment = temp;
                        clear temp;
                        residueposition(alk+2:end) = residueposition(alk+2:end)  - 1; % since we have deleted one column
                    end                
                end
                residueposition(alk+1) = residueposition(alk)+mgaps;
            elseif (residueposition(alk+1) - residueposition(alk)) ...
                    < (pdbRes{i}.number(alk+1) - pdbRes{i}.number(alk))
                % something wrong in the alignment, need to move the position
                % of the alk+1 residue next to the alk+2 residue
                mgaps =  (pdbRes{i}.number(alk+2) - pdbRes{i}.number(alk+1));
                if Alignment(1, residueposition(alk+2)-mgaps) == '-'
                    Alignment(1, residueposition(alk+2)-mgaps) = Alignment(1, residueposition(alk+1));
                    if Alignment(3, residueposition(alk+2)-mgaps) == Alignment(1, residueposition(alk+2)-mgaps)
                        Alignment(2, residueposition(alk+2)-mgaps) = '|';
                    else
                        if Alignment(2, residueposition(alk+2)) == '|' 
                            % if the next one is a match, this one is miss
                            % match residue
                            Alignment(2, residueposition(alk+2)-mgaps) = ':';
                        else
                            % otherwise, a space needs to be inserted into the
                            % third sequence
                            % insert before residueposition(alk+2)-1 on the
                            % first sequence
                            % insert after residueposition(alk+2)-1 on the
                            % third sequence          
                            clear temp1;
                            temp1 = [Alignment(1, 1:residueposition(alk+2)-mgaps-1),'-', Alignment(1, residueposition(alk+2)-mgaps:end)];
                            clear temp3;
                            temp3 = [Alignment(3, 1:residueposition(alk+2)-mgaps),'-', Alignment(3, residueposition(alk+2):end)];
                            clear temp2;
                            temp2 = [Alignment(2, 1:residueposition(alk+2)-mgaps-1),'--', Alignment(2, residueposition(alk+2):end)];                            
                            Alignment = [temp1;temp2;temp3];
                        end                        
                    end
                    Alignment(1, residueposition(alk+1)) = '-';
                    if Alignment(3, residueposition(alk+1)) ~= '-'
                        Alignment(2, residueposition(alk+1)) = ' ';
                    else
                        clear temp;
                        temp = Alignment(:,[1:residueposition(alk+1)-1,residueposition(alk+1)+1:end]);
                        clear Alignment;
                        Alignment = temp;
                        clear temp;
                        residueposition(alk+2:end) = residueposition(alk+2:end)  - 1; % since we have deleted one column
                    end
                end
                residueposition(alk+1) = residueposition(alk+2)-mgaps;
            end
        end
    end   
%     Alignment
%     pause();
    %% assign the resNum
    residue{i}.number = [];
    residue{i}.chain = [];    
    t = 0;
    for j = 1 : size(Alignment,2)
        if Alignment(1,j) ~= '-'
            t = t + 1;
            if Alignment(3,j) ~= '-'
                if Alignment(2,j) ~= '|';
                    % miss match
                    residue{i}.number = [residue{i}.number, -199999];
                    residue{i}.chain = [residue{i}.chain, pdbRes{i}.chain(t)];
                else
                    % match
                    residue{i}.number = [residue{i}.number, pdbRes{i}.number(t)];
                    residue{i}.chain = [residue{i}.chain, pdbRes{i}.chain(t)];
                end
            end
        else
            % missing residues in pdb
            if Alignment(3,j) ~= '-'
                residue{i}.number = [residue{i}.number, -299999];
                residue{i}.chain = [residue{i}.chain, pdbRes{i}.chain(1)];    
            end
        end
    end   
    % find first match residue
    k = 1;
    while residue{i}.number(k) == -199999  ||  residue{i}.number(k) == -299999
        k = k + 1;        
    end
    if k > 1
        % change the resNum for the extra residues
        if residue{i}.number(k) == min(pdbRes{i}.number) 
            for s = 1 : k-1
                residue{i}.number(s) = min(pdbRes{i}.number) - (k-1) + (s-1);
            end
        else      
            keyyy = 1;
            for s = 1 : k-1       
                if residue{i}.number(s) ~= -299999
                    keyyy =  0;
                end                
            end
            if keyyy == 1 % all the extra residues at the begining are missing residue
                for s = 1 : k-1
                    residue{i}.number(s) = residue{i}.number(k)  -(k-1) + (s-1) ;
                end
            else
               disp( ['the residues at the begining of the reference seqeunce mismatch the corresponding residues in the pdb for protein ', num2str(i)]);
               disp('please check the alignment, and change the mismatch residues in the reference sequence to match that in the pdb')
               disp('ctrl+c stop the program')
               pause();
            end
        end        
    end
    % find the last match residue
    k = size(residue{i}.number,2);
    while  residue{i}.number(k) == -199999  ||  residue{i}.number(k) == -299999
        k = k - 1;
    end
    if k < size(residue{i}.number,2)
        if residue{i}.number(k) == max(pdbRes{i}.number) 
            for s = k+1:size(residue{i}.number,2)
                residue{i}.number(s) = max(pdbRes{i}.number) + s-k;
            end
        else  
            keyyy = 1;
            for s = k+1:size(residue{i}.number,2)
                if residue{i}.number(s) ~= -299999
                    keyyy =  0;
                end                
            end
            if keyyy == 1  % all the extra residues at the begining are missing residue
                 for s = k+1:size(residue{i}.number,2)
                    residue{i}.number(s) = residue{i}.number(k) + s-k;
                end
%             for s = size(residue{i}.number,2):-1:k+1
%                 residue{i}.number(s) = 99999 + s;
%             end
            else
                disp( ['the residues at the end of the reference seqeunce mismatch the corresponding residues in the pdb for protein ', num2str(i)]);
                disp('please check the alignment, and change the mismatch residues in the reference sequence to match that in the pdb')
                disp('ctrl+c stop the program')
                pause();
            end
        end
    end
    for k = 1 : size(residue{i}.number,2)        
        % missing residues
        if residue{i}.number(k) == -299999
            % in the middle
            if k-1 > 0 && k < size(residue{i}.number,2)
                if abs(residue{i}.number(k+1) - residue{i}.number(k-1)) > 1
                     residue{i}.number(k) =  residue{i}.number(k-1) + 1;
                end
            end            
        end        
    end
    if size(find(residue{i}.number == -199999),2) +  size(find(residue{i}.number == -299999), 2) > 0
        'wrong in residue number assign'
        pause();
    end
    allAln{i} = Alignment;
end
fid = fopen('alignmentforresseqassign.txt','w');
fprintf(fid, '%s\n', ['These are the alignments of the reference sequences',...
    'and the sequences from the pdb files. The coordinates for',...
    'each residue in the pdb file will be assign to the',...
    'corresponding positions in the MSA according to this alignment.']);
disp(['Please do check the alignment(alignmentforresseqassign.txt)',...
    'to make sure nothing odd had happened. Please also note that we are using the continuous domains for the residue number assignment of the noncontiuous domains']);
fprintf(fid, '%s\n', ['if something wrong with the alignment',...
     'please change the alignment by editing the',...
     'Alignment.mat and specify the manualalignment = 1 to use the manual alignment, rerun the programe']);
for i = 1 : 2
    fprintf(fid,['For protein ', num2str(i), '\n']);
    for j = 1 : size(allAln{i},1)
        fprintf(fid, '%s\n', allAln{i}(j,:));
    end
end
fclose(fid);
save('Alignment.mat', 'allAln');
%% resNum Check1: check if the resNum is sequentially increasing or not
for i = 1 : 2
    for j = 2 : size(residue{i}.number,2)
        if residue{i}.number(j) > -1000 && residue{i}.number(j-1) > -1000
            if residue{i}.number(j) < residue{i}.number(j-1)
                disp(['Error: error in residue number assignment. Protein ', num2str(i),' ', num2str(j), 'th resNum is smaller than',...
                    'the previous residue']);
                disp('might be something wrong with the alignment between the pdb sequence and the reference sequence');
                disp('stop the programe by ctrl+c and check the alignment, make your own alignment if neccessary');
                disp(['if something wrong with the alignment',...
                    '1) please change the alignment by editing the',...
                    'Alignment.mat and save it as [jobname, myAlignment.mat]',...
                    '2) At the same time, specify the parameters.manualalignment = 1 in Demo.m',...
                    'to use the manual alignment, rerun the programe']);
                pause();
            end
        end
    end
end

%% resNum Check3: if every residue on the reference sequence has got a residue number
for i = 1 : 2
    length =location{i}(size(location{i},1),2) - location{i}(1,1)+1;
    if size(residue{i}.number,2) ~= length
        'Error: error in residue number assignment, not every residue on the sequence has  its residue number in assign_resSeq2refseq.m line 241'
        pause();
    end
end

%% for non-continuous domains
%% need to put the residue numbers for one domain together
for i = 1 : 2
    if size(location{i},1) > 1
        % non-continuous domain
        clear location1;
        location1 = location{i} - location{i}(1,1) + 1;
        clear residue1;
        residue1.number = [];
        residue1.chain = [];
        for j = 1 : size(location1,1)
            residue1.number = [residue1.number, residue{i}.number([location1(j,1) : location1(j,2)])];
            residue1.chain =[residue1.chain, residue{i}.chain([location1(j,1) : location1(j,2)])];
        end
        residue{i}.number = [];
        residue{i}.chain = [];
        residue{i}.number = residue1.number;
        residue{i}.chain = residue1.chain ;
    end
end


    
    
% recover the full sequence for the local MSA
function newAln = expandlocalAln(pdbSeq, sequence, oldAln)
boundary(1,:) = findboundary(pdbSeq, oldAln(1, oldAln(1,:)~='-'));
boundary(2,:) = findboundary(sequence, oldAln(3, oldAln(3,:)~= '-'));
t = 1;
for i = 1 : boundary(1,1)-1
    newAln(1,t) = pdbSeq(i);
    newAln(2,t) = '-';
    newAln(3,t) = '-';
    t = t + 1;
end
for i = 1 : boundary(2,1)-1
    newAln(1,t) = '-';
    newAln(2,t) = '-';
    newAln(3,t) = sequence(i);
    t = t + 1;
end
for i = 1 : size(oldAln,2)
    newAln(1,t) = oldAln(1,i);
    newAln(2,t) = oldAln(2,i);
    newAln(3,t) = oldAln(3,i);
    t = t + 1;
end
for i =  boundary(2,2)+1 : size(sequence,2)
    newAln(1,t) = '-';
    newAln(2,t) = '-';
    newAln(3,t) = sequence(i);
    t = t + 1;
end
for i = boundary(1,2)+1 : size(pdbSeq,2)
    newAln(1,t) = pdbSeq(i);
    newAln(2,t) = '-';
    newAln(3,t) = '-';
    t = t + 1;
end
% find boundary of the local msa on the full sequence
function boundary = findboundary(fullseq, localseq)
ll = size(localseq,2);
for j = 1 : size(fullseq,2)-ll+1
    if fullseq(j:j+ll-1) == localseq
        boundary(1,1) = j;
        boundary(1,2) = j+ll-1;
        break;
    end
end



