% % make a new aln file for the method
% 1. put the columns for one domain together
% 2. delete the columns with too many gaps
% 3. calculate the intradomain distances for each of the domains and
% generate the intradomain neighbor information for each column in the MSA

% input: protein structure
% fields for protein structure: names, gi_number, alnfile_name,
% domnumfile_num, infile, structure, aln.seq, aln.domnum, aln.resnum
% output: write a file as aln for scoring system
%         seprate_site is the length of the first domain

function protein2 = preprocess_aln_file(protein, parameters, temppath)
protein2 = protein;
%% put the columns for each domain together 
location = protein.aln.locationMSA;
residue = protein.aln.residue;
domain = protein.aln.domain;
protein2.aln.new.residue.number = [];
protein2.aln.new.residue.chain = [];
% map the domain bounds with residue number on the MSA
t = 1;
for i = 1 : size(domain.number,2)
    for j = 1 : size(domain.number{i},1)
        for k = 1 : 2
            domainmapped(t) = get_locationMSA_from_resnum(location,...
                residue, domain.number{i}(j,k), domain.chain{i});
            if domainmapped(t) == 0
                fid = fopen('domaindefinitionmodified.txt', 'a');
                fprintf(fid, '%s:  %d', protein.names, domain.number{i}(j,k));
                'error in domain re-arrangement of columns in sequence'
                'we need to modify the domain definition'
                while ~domainmapped(t) 
                    if k == 1
                        domain.number{i}(j,k) = domain.number{i}(j,k) + 1;                        
                    else
                        domain.number{i}(j,k) = domain.number{i}(j,k) - 1;
                    end
                    domainmapped(t) = get_locationMSA_from_resnum(location,...
                        residue, domain.number{i}(j,k), domain.chain{i});
                end
                fprintf(fid, '=> %d \n', domain.number{i}(j,k));
                fclose(fid);
                protein2.aln.domain = domain;                    
            end
            t = t + 1;
        end
        % new_resnum      
        for jk = 1 : domain.number{i}(j,2)-domain.number{i}(j,1)+1
            clear restemp;
            restemp.number = domain.number{i}(j,1)+jk-1;
            restemp.chain = domain.chain{i};
            if isInResidueSet(restemp, protein2.aln.residue)
                protein2.aln.new.residue.number = [protein2.aln.new.residue.number, ...
                    restemp.number];
                protein2.aln.new.residue.chain  = [protein2.aln.new.residue.chain, ...
                    restemp.chain];
            end
        end
    end
end
% rearrange the columns into domains only for two domains
length = protein.aln.length;
exclusivelocation = [];
for i = 1 : length
    if isin(i, location)
        % has a residue
        clear restemp;
        restemp = get_resnum_from_locationMSA(location, residue, i);
        if ~isInResidueSet(restemp, protein2.aln.new.residue)
            % if this resSeq is not in the new residue set, it must be a
            % miss match residue between the reference sequence and the pdb
            % sequence. we cannot use the structure information
            exclusivelocation = [exclusivelocation, i];
        end
    end
end
index_domain1 = [];
index_domain2 = [];
for i = 1 : length
    if ~isin(i, exclusivelocation)
        t = 1;
        for j = 1 : size(domain.number{1},1)
            if i > domainmapped(t)-1 && i < domainmapped(t+1)+1
                index_domain1 = [index_domain1, i];
            end
            t = t + 2;
        end
        for j = 1 : size(domain.number{2},1)
            if i > domainmapped(t)-1 && i < domainmapped(t+1)+1
                index_domain2 = [index_domain2, i];
            end
            t = t + 2;
        end
    end
end
protein2.aln.new.seprate_site = size(index_domain1,2);
clear index;
index = [index_domain1, index_domain2];
for i = 1 : size(protein.aln.seq,1)     
    protein2.aln.new.seq(i,1).Header = protein.aln.seq(i).Header;
    protein2.aln.new.seq(i,1).Sequence = protein.aln.seq(i).Sequence(index);
end
protein2.aln.new.locationMSA = find_location(protein2.aln.new.seq, protein.gi_number);

filename = [temppath, protein.names, '.fasta'];
fastawrite(filename, protein2.aln.new.seq);
protein2.aln.new.length = size(protein2.aln.new.seq(1).Sequence,2);

%% pre-processing with the MSA: deleting the GAPs
'deleting columns with too many gaps...'
threshold_gaps = parameters.gapcutoff;
filename_nogap = protein.infile;
protein2.aln.nogap.seq = delete_gaps(filename, filename_nogap, threshold_gaps);
load([filename_nogap,'_loc.mat']); % left locations
protein2.aln.nogap.length = size(left_column_vector,2);
protein2.aln.nogap.residue = get_resnum_from_locationMSA(protein2.aln.new.locationMSA, ...
    protein2.aln.new.residue, left_column_vector);
protein2.aln.nogap.locationMSA = find_location(protein2.aln.nogap.seq, protein.gi_number);
protein2.aln.nogap.seprate_site = find_seprate_site(protein2.aln.nogap.locationMSA,...
    protein.aln.domain, protein2.aln.nogap.residue);


%% delete the columns without structure information
clear temp;
temp = compute_atom_distances(protein.structure, protein2.aln.nogap.residue);
protein2.aln.nogap.distances = temp.distances;
left_vec = temp.left_vec; % left indexes of residues
protein2.aln.nogap.structure.length = size(left_vec, 2);
protein2.aln.nogap.structure.residue.number = protein2.aln.nogap.residue.number(left_vec);
protein2.aln.nogap.structure.residue.chain = protein2.aln.nogap.residue.chain(left_vec);
left_sequence_vec = get_locationMSA_from_resnum(protein2.aln.nogap.locationMSA, protein2.aln.nogap.residue,...
    protein2.aln.nogap.structure.residue.number, protein2.aln.nogap.structure.residue.chain);
for j = 1 : size(left_sequence_vec,2)
    if left_sequence_vec(j) == 0
        disp(['error in delete columns without structure infromation in preprocess_aln_file.m', protein2.names]);
        pasue();
    end
end
for j = 1 : size(protein2.aln.nogap.seq, 1)
    protein2.aln.nogap.structure.seq(j,1).Header =  protein2.aln.nogap.seq(j).Header;
    protein2.aln.nogap.structure.seq(j,1).Sequence = protein2.aln.nogap.seq(j).Sequence(left_sequence_vec);
end
protein2.aln.nogap.structure.locationMSA = find_location(protein2.aln.nogap.structure.seq, protein.gi_number);
protein2.aln.nogap.structure.seprate_site = find_seprate_site(protein2.aln.nogap.structure.locationMSA, ...
    protein.aln.domain, protein2.aln.nogap.structure.residue);
% fastawrite(protein.infile_nogap_structure, protein2.aln.nogap.structure.seq);

'generating indexes for amino acids in MSA ....'
%% save the indexMSA20AA
num_seq = size(protein2.aln.nogap.seq,1);
num_column = size(protein2.aln.nogap.seq(1).Sequence,2);
alignment = protein2.aln.nogap.seq;
aminonames = {'R', 'N', 'D', 'C', 'Q', 'E', 'G', 'H', 'I', 'L', 'K', 'M', 'F', 'P', 'S', 'T', 'W', 'Y', 'V', 'A'};
for i = 1 : num_seq
    for j = 1 : num_column
        mapped_index20(i,j) = MapResiduesToIndex20(alignment(i,1).Sequence(j), aminonames);
    end
end
save([temppath, 'IndexMSA20AA', protein2.names,'.mat'], 'mapped_index20');
for i = 1 : num_seq
    for j = 1 : num_column
        mapped_index(i,j) = MapResiduesToIndex(alignment(i,1).Sequence(j), aminonames);
    end
end
save([temppath, 'IndexMSA', protein2.names,'.mat'], 'mapped_index');

%% calculate the absEntropy for each column in the nogap MSA
protein2.aln.nogap.absEntropy = compute_absEntropy(mapped_index20);


